from zope.interface import implements
from zope.component import getMultiAdapter
from Products.Five import BrowserView
from plone.memoize import view
from Products.CMFPlone.utils import safe_unicode
from zope.i18n import translate

from Products.PasswordResetTool.interfaces import IPasswordResetToolView
from Products.PasswordResetTool import passwordresetMessageFactory as _
from email.Header import Header

from plone.registry.interfaces import IRegistry
from zope.component import getUtility
try:
    from Products.CMFPlone.interfaces.controlpanel import IMailSchema
    HAS_REGISTRY_MAIL_SETTINGS = True
    # work with plone 4 yet...
except ImportError:
    HAS_REGISTRY_MAIL_SETTINGS = False


class PasswordResetToolView(BrowserView):
    implements(IPasswordResetToolView)

    @view.memoize_contextless
    def portal_state(self):
        """ returns
            http://dev.plone.org/plone/browser/plone.app.layout/trunk/plone/app/layout/globals/portal.py
        """
        return getMultiAdapter((self.context, self.request), name=u"plone_portal_state")

    def encode_mail_header(self, text):
        """ Encodes text into correctly encoded email header """
        return Header(safe_unicode(text), 'utf-8')

    def encoded_mail_sender(self):
        """ returns encoded version of Portal name <portal_email> """
        if HAS_REGISTRY_MAIL_SETTINGS:
            registry = getUtility(IRegistry)
            mail_settings = registry.forInterface(IMailSchema, prefix="plone")
            from_ = mail_settings.email_from_name
            mail = mail_settings.email_from_address
        else:
            portal = self.portal_state().portal()
            from_ = portal.getProperty('email_from_name')
            mail = portal.getProperty('email_from_address')
        return '"%s" <%s>' % (self.encode_mail_header(from_), mail)

    def registered_notify_subject(self):
        portal_name = self.portal_state().portal_title()
        return translate(_(u"mailtemplate_user_account_info",
                           default=u"User Account Information for ${portal_name}",
                           mapping={'portal_name': safe_unicode(portal_name)}),
                           context=self.request)

    def mail_password_subject(self):
        return translate(_(u"mailtemplate_subject_resetpasswordrequest",
                           default=u"Password reset request"),
                           context=self.request)



"""Initialize PasswordResetTool Product"""

from Products.CMFCore import utils
import PasswordResetTool
from zope.i18nmessageid import MessageFactory
passwordresetMessageFactory = MessageFactory('passwordresettool')

tools = (PasswordResetTool.PasswordResetTool, )


def initialize(context):
    utils.ToolInit('Password Reset Tool',
                    tools=tools,
                    icon='tool.gif'
                    ).initialize(context)



"""
PasswordResetTool doctests
"""

import doctest
import unittest

from Acquisition import aq_base
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost
from plone.app import testing
from plone.registry.interfaces import IRegistry
from plone.testing import layered
from transaction import commit
from zope.component import getSiteManager
from zope.component import getUtility

try:
    from Products.CMFPlone.interfaces.controlpanel import IMailSchema, ISiteSchema
    HAS_REGISTRY_MAIL_SETTINGS = True
    # work with plone 4 yet...
except ImportError:
    HAS_REGISTRY_MAIL_SETTINGS = False


OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_ONLY_FIRST_FAILURE)


class MockMailFixture(testing.PloneSandboxLayer):

    defaultBases = (testing.PLONE_FIXTURE, )

    def setUpPloneSite(self, portal):
        portal._original_MailHost = portal.MailHost
        portal.MailHost = mailhost = MockMailHost('MailHost')
        registry = getUtility(IRegistry)
        mail_settings = registry.forInterface(IMailSchema, prefix='plone')
        mail_settings.smtp_host = u'localhost'
        mail_settings.email_from_address = 'john@doe.com'
        sm = getSiteManager(context=portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        portal.email_from_address = 'test@example.com'
        commit()

    def tearDownPloneSite(self, portal):
        portal.MailHost = portal._original_MailHost
        sm = getSiteManager(context=portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(
            aq_base(portal._original_MailHost),
            provided=IMailHost
        )


class Layer(testing.FunctionalTesting):

    def set_email_from_name(self, name):
        if HAS_REGISTRY_MAIL_SETTINGS:
            registry = getUtility(IRegistry)
            mail_settings = registry.forInterface(IMailSchema, prefix="plone")
            mail_settings.email_from_name = name
        else:
            self['portal'].email_from_name = name

    def set_email_from_address(self, address):
        if HAS_REGISTRY_MAIL_SETTINGS:
            registry = getUtility(IRegistry)
            mail_settings = registry.forInterface(IMailSchema, prefix="plone")
            mail_settings.email_from_address = address
        else:
            self['portal'].email_from_address = address

    def set_portal_title(self, name):
        if HAS_REGISTRY_MAIL_SETTINGS:
            registry = getUtility(IRegistry)
            site_settings = registry.forInterface(ISiteSchema, prefix='plone')
            site_settings.site_title = name
        else:
            self['portal'].title = name

MOCK_MAIL_FIXTURE = MockMailFixture()
MM_FUNCTIONAL_TESTING = Layer(
    bases=(MOCK_MAIL_FIXTURE, ),
    name='PloneTestCase:Functional'
)


def test_suite():
    return unittest.TestSuite((
        layered(
            doctest.DocFileSuite(
                'browser.txt',
                optionflags=OPTIONFLAGS,
                package='Products.PasswordResetTool.tests',
            ),
            layer=MM_FUNCTIONAL_TESTING
        ),
        layered(
            doctest.DocFileSuite(
                'view.txt',
                optionflags=OPTIONFLAGS,
                package='Products.PasswordResetTool.tests',
            ),
            layer=MM_FUNCTIONAL_TESTING))
        )



from setuptools import setup, find_packages

version = '2.2.0'

setup(name='Products.PasswordResetTool',
      version=version,
      description="Password reset tool for Plone",
      long_description=open("README.txt").read() + "\n" + open("CHANGES.txt").read(),
      classifiers=[
          "Framework :: Plone",
          "Framework :: Plone :: 5.0",
          "Framework :: Zope2",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
      ],
      keywords='password reset plone',
      author='J. Cameron Cooper',
      author_email='prt@jcameroncooper.com',
      url='http://pypi.python.org/pypi/Products.PasswordResetTool',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      extras_require=dict(
          test=[
              'plone.app.testing',
          ]
      ),
      install_requires=[
          'setuptools',
          'plone.memoize',
          'zope.component',
          'zope.i18nmessageid',
          'zope.i18n',
          'zope.interface',
          'Products.CMFPlone',
          'Products.CMFCore',
          'Acquisition',
          'DateTime',
          'Zope2',
          'plone.app.registry'
      ],
      )
